@@ -7,13 +7,13 @@ data that was previously present in the MMIF file.
 """
 
 from datetime import datetime
-from typing import Dict, Union, Optional
+from typing import Dict, Union, Optional, Generator
 import dateutil.parser
-from pyrsistent import pmap
+from pyrsistent import pmap, pvector
 
 from .annotation import Annotation, Document
 from .model import FreezableMmifObject, FreezableDataList, FreezableDataDict
-from mmif.vocabulary import AnnotationTypesBase
+from mmif.vocabulary import ThingTypesBase
 
 
 __all__ = ['View', 'ViewMetadata', 'Contain']
@@ -41,9 +41,13 @@ class View(FreezableMmifObject):
             'metadata': ViewMetadata,
             'annotations': AnnotationsList
         })
+        self._required_attributes = pvector(["id", "metadata", "annotations"])
         super().__init__(view_obj)
+        for item in self.annotations:
+            if isinstance(item, Document):
+                item.parent = self.id
 
-    def new_contain(self, at_type: Union[str, AnnotationTypesBase], contain_dict: dict = None) -> Optional['Contain']:
+    def new_contain(self, at_type: Union[str, ThingTypesBase], contain_dict: dict = None) -> Optional['Contain']:
         """
         Adds a new element to the ``contains`` metadata.
 
@@ -53,7 +57,7 @@ class View(FreezableMmifObject):
         """
         return self.metadata.new_contain(at_type, contain_dict)
 
-    def new_annotation(self, aid: str, at_type: Union[str, AnnotationTypesBase], overwrite=False) -> 'Annotation':
+    def new_annotation(self, aid: str, at_type: Union[str, ThingTypesBase], overwrite=False) -> 'Annotation':
         """
         Generates a new :class:`mmif.serialize.annotation.Annotation`
         object and adds it to the current view.
@@ -106,8 +110,31 @@ class View(FreezableMmifObject):
                           an existing view with the same ID
         :return: None
         """
+        document.parent = self.id
         return self.add_annotation(document, overwrite)
 
+    def get_annotations(self, at_type: Union[str, ThingTypesBase] = None, **properties) -> Generator[Annotation, None, None]:
+        """
+        Look for certain annotations in this view, specified by parameters
+
+        :param at_type: @type of the annotations to look for. When this is None, any @type will match.
+        :param properties: properties of the annotations to look for.
+        When given more than one property, all properties must match.
+        Note that some annotation properties (annotation type medata) are specified in the `contains` view metadata
+        """
+        def prop_check(k, v, *props):
+            return any(k in prop and prop[k] == v for prop in props)
+
+        if at_type:
+            at_type_metadata = self.metadata.contains[str(at_type)]
+        for annotation in self.annotations:
+            if not at_type:
+                if all(map(lambda kv: prop_check(kv[0], kv[1], annotation.properties, self.metadata.contains[str(annotation.at_type)]), properties.items())):
+                    yield annotation
+            if at_type and str(annotation.at_type) == str(at_type):
+                if all(map(lambda kv: prop_check(kv[0], kv[1], annotation.properties, at_type_metadata), properties.items())):
+                    yield annotation
+
     def get_documents(self):
         return [annotation for annotation in self.annotations if annotation.is_document()]
 
@@ -154,6 +181,7 @@ class ViewMetadata(FreezableMmifObject):
         self.timestamp: Optional[datetime] = None
         self.app: str = ''
         self.contains: ContainsDict = ContainsDict()
+        self._required_attributes = pvector(["app", "contains"])
         super().__init__(viewmetadata_obj)
 
     def _deserialize(self, input_dict: dict) -> None:
@@ -189,7 +217,7 @@ class ViewMetadata(FreezableMmifObject):
                 break
         return exists
 
-    def new_contain(self, at_type: Union[str, AnnotationTypesBase], contain_dict: dict = None) -> Optional['Contain']:
+    def new_contain(self, at_type: Union[str, ThingTypesBase], contain_dict: dict = None) -> Optional['Contain']:
         """
         Adds a new element to the ``contains`` dictionary.
 
@@ -197,7 +225,7 @@ class ViewMetadata(FreezableMmifObject):
         :param contain_dict: any metadata associated with the annotation type
         :return: the generated :class:`Contain` object
         """
-        if isinstance(at_type, AnnotationTypesBase):
+        if isinstance(at_type, ThingTypesBase):
             exists = self._find_match_hotfix(at_type.name) or self._find_match_hotfix(at_type.value)
             final_key = at_type.value
         else:
@@ -206,7 +234,6 @@ class ViewMetadata(FreezableMmifObject):
 
         if not exists:
             new_contain = Contain(contain_dict)
-            new_contain.gen_time = datetime.now()
             self.contains[final_key] = new_contain
             return new_contain
 
